@echo off
setlocal
set _DEFAULT_PORT=COM70

:: first handle no parameters as %Var:...=...% barfs on unset variables. 
if %1.==. goto :skiphlp
:: Is help wanted?
set p1=%1
call :toUpper p1
set p1=%p1:--=-%
set p1=%p1:/=-% 
if %p1%==-? goto :help
if %p1%==-H goto :help
if %p1%==-HELP goto :help
:skiphlp

:: is it running as Administrator?
fsutil>nul dirty query %systemdrive%
if errorlevel 1 set "err=SetupRobotLink.bat must be run as Administrator" &goto :error

:: Find installed com0com folder from registry
call :getRegVal C0C "HKLM\SOFTWARE\com0com" "Install_Dir" 32
if errorlevel 1 set "err=com0com is not installed" &goto :error
for %%a in (com0com.sys com0com.inf setupc.exe) do if not exist "%C0C%\%%a" set "err=COM0COM is missing a critical file "%%a"" &goto :error
if not exist "%C0C%\com2tcp.exe" set "err="com2tcp.exe" must be in the com0com folder" &goto :error


if not exist "%~p0shortcut.exe" set "err=Shortcut utility not found, can't create launcher" &goto :error

if %1.==. set "err=espLink network name or IP address must be specified" &goto :error

set p2=%~2
if "%p2%"=="" set p2=%_DEFAULT_PORT% & goto :isCOMn
:: Validate port
call :toUpper p2
:: match to REGEX :start: C O M non-zero_digit opt_digit(s) :end:
echo %p2%|findstr>nul /r "^COM[1-9][0-9]*$"
if errorlevel 1  set "err=%P2% is not a COM port" &goto :error

:isCOMn
:: check if reusing the port, yes: skip other checks
call :getc0cnum e ESPLINK
call :getc0cnum c %p2%
if "%e%"=="%c%" goto :doSetup

:: check if port is available
"%C0C%\setupc" busynames COM* |%SystemRoot%\System32\find>nul "%p2%"
if not errorlevel 1 set "err=%P2% already exists" &goto :error

:doSetup
echo:
echo SetupRobotLink is configuring ports ...
:: setupc needs to 'see' its inf files to modify anything
:: so need to change to its directory.
pushd %C0C%
:: Remove ESPLINK port pair if present
if not %e%.==. setupc>nul remove %e%
:: Create new port pair (reusing number if possible)
setupc>nul --wait +5 install %e% PortName=COM#,PlugInMode=yes,HiddenMode=no PortName=ESPLINK,PlugInMode=no,HiddenMode=yes
:: 'freshen' pair number
call :getc0cnum e ESPLINK
:: can't set RealPortName at install time, so do it now
setupc>nul change CNCA%e% RealPortName=%p2%
popd

echo Done.  It is now creating the daemon launcher ...
"%~p0shortcut">nul /a:c /f:"Connect to robot.lnk" /t:"cmd" /w:. /p:"/c \"%C0C%\com2tcp.exe\" --baud 115200 \\.\ESPLINK %~1 2323 &pause" /r:7 /d:"Link virtual (com0com) COM port to robot.  Close to unlink." /i:%SystemRoot%\System32\pifmgr.dll,27
if errorlevel 1 set "err=Failed to create the daemon launcher" &goto :error

Echo All done.
endlocal
goto :eof

:: ************************* Subroutines *****************************

:help  -- displays help text
@echo off
echo:
echo +----------------------------------------------+
echo ^| SetupRobotLink com0com Configuration Utility ^|
echo +----------------------------------------------+
echo:
echo SetupRobotLink.bat sets up a COM0COM port pair to connect to the robot over
echo WiFi as if its Arduino was directly connected to the PC and creates a launcher
echo for the COM2TCP telnet link daemon to connect to the robot.
echo:
echo Usage:
echo   SetupRobotLink ^<RobotHostName^> {COM^<n^>}
echo:
echo ^<RobotHostName^> is the host name or IP address of the robot's espLink.
echo:
echo COM^<n^> is the virtual COM port for use in the Arduino IDE.  If not specified,
echo it defaults to COM70. The port must be available.
echo:
echo The daemon launcher is created as "Connect to robot.lnk" in the current folder.
echo Running it makes the virtual com port appear, and closing it hides the port.
echo Other robots can be connected (one at a time) without rerunning this utility by
echo editing the robot name in the properties of a copy of the launcher ^(near the
echo end of the command, between ESPLINK and 2323^).
echo:
echo com0com (and com2tcp) can be found at: https://sourceforge.net/projects/com0com/
echo espLink can be found at: https://github.com/jeelabs/esp-link
echo Shortcut utility can be found at: https://www.optimumx.com/downloads.html

:: Pause if batch file directly clicked from explorer
if defined SESSIONNAME echo: &pause
endlocal
EXIT /B 0

:error
echo: 1>&2
echo 1>&2   Error: %err%.  For help, try: %0 /?
if defined SESSIONNAME echo: &pause

endlocal
EXIT /B 1

:getRegVal Var RegKey ValueName {Mode} -- Reads the registry and returns the contents of FullKey ValueName 
::                      -- Var [out] - name of result variable
::                      -- RegKey [in] - Registry key containing ValueName
::                      -- ValueName - the Registry value to read
::			-- Mode [in,opt] - if present, either 32 or 64 to force a registry view  
::                      If ValueName is found, Var is set to Registry data, else Var becomes unset.

:: Prepare to return with Var unset
set %~1=
:: if the RegKey or ValueName is null, return errorlevel 2
if %~2.==. exit /b 2
if %~3.==. exit /b 2 

setlocal
:: build cmd string
set "_cmd=reg query "%~2^" /V ^"%~3^"" 
if %~4.==32. set "_cmd=%_cmd% /reg:32"
if %~4.==64. set "_cmd=%_cmd% /reg:64"
set _data=
:: parse result of reg query to get data.
for /f "usebackq tokens=2,*" %%a in (`"%_cmd%" 2^>nul`) do echo %%a|%SystemRoot%\System32\find>nul "REG_" &&set _data="%%~b"
:: if not found return errorlevel 1
if %_data%.==. endlocal &exit /b 1
:: strip quotes (from REG_SZ value)
for /f "usebackq tokens=*" %%a in ('%_data%') do set _data=%%~a
:: return data in Var with errorlevel 0
endlocal &set %~1=%_data%
EXIT /b 0

:getc0cnum Var PortName -- Find which c0c port pair is using PortName
::                      -- Var [out] - name of result variable
::                      -- PortName [in] - Port name to find
::                      If found, Var is set to the number of the port pair, else Var becomes unset.
::			The search is case insensitive.
::                      Requires %C0C% to be set to c0c program folder

:: Prepare to return with Var unset
set %~1=
:: if the portname is null, return errorlevel 2
if %~2.==. exit /b 2 
setlocal
set _result=
set _setupc="%C0C%\setupc"

:: Parse result of C0C setupc list into raw port (%%a) and rest of line (%%b)
:: then search in %%b for PortName (%2) and if found, set result to raw port.                            
for /f "usebackq tokens=1,* delims= " %%a in (`%_setupc% list`) do (echo %%b|%SystemRoot%\System32\find>nul /i "PortName=%~2,") &&set _result=%%a
:: if not found return errorlevel 1
if %_result%.==. endlocal &exit /b 1
:: trim result to just the number and return it in Var with errorlevel 0
endlocal &set %~1=%_result:~4%
EXIT /b 0

:toUpper str -- converts lowercase character to uppercase
::           -- str [in,out] - valref of string variable to be converted
:$created 20060101 :$changed 20080219 :$categories StringManipulation
:$source https://www.dostips.com
if not defined %~1 EXIT /b
for %%a in ("a=A" "b=B" "c=C" "d=D" "e=E" "f=F" "g=G" "h=H" "i=I"
            "j=J" "k=K" "l=L" "m=M" "n=N" "o=O" "p=P" "q=Q" "r=R"
            "s=S" "t=T" "u=U" "v=V" "w=W" "x=X" "y=Y" "z=Z" "="
            "=" "=") do (
    call set %~1=%%%~1:%%~a%%
)
EXIT /b
